@@ -96,6 +96,8 @@ class Addressee::Private : public QSharedData
       mAddresses = other.mAddresses;
       mKeys = other.mKeys;
       mEmails = other.mEmails;
+      mLangs =  other.mLangs;
+      mGender = other.mGender;
       mCategories = other.mCategories;
       mCustomFields = other.mCustomFields;
 
@@ -140,7 +142,10 @@ class Addressee::Private : public QSharedData
     PhoneNumber::List mPhoneNumbers;
     Address::List mAddresses;
     Key::List mKeys;
-    QStringList mEmails;
+    Email::List mEmails;
+    Lang::List mLangs;
+    Gender mGender;
+    QString mKind;
     QStringList mCategories;
     QMap<QString, QString> mCustomFields;
 
@@ -337,7 +342,7 @@ bool Addressee::operator==( const Addressee &addressee ) const
     return false;
   }
 
-  if ( !emailsEquals( d->mEmails, addressee.d->mEmails ) ) {
+  if ( !listEquals( d->mEmails, addressee.d->mEmails ) ) {
     kDebug() << "emails differs";
     return false;
   }
@@ -352,6 +357,19 @@ bool Addressee::operator==( const Addressee &addressee ) const
     return false;
   }
 
+  if (d->mLangs != addressee.d->mLangs) {
+      kDebug() << "langs differs";
+      return false;
+  }
+  if (d->mGender != addressee.d->mGender) {
+      kDebug() << "gender differs";
+      return false;
+  }
+  if ( d->mKind != addressee.d->mKind ) {
+    kDebug() << "kind differs";
+    return false;
+  }
+
   return true;
 }
 
@@ -403,6 +421,20 @@ QString Addressee::nameLabel()
   return i18n( "Name" );
 }
 
+void Addressee::setKind( const QString &kind )
+{
+  if ( kind == d->mKind )
+    return;
+
+  d->mEmpty = false;
+  d->mKind = kind;
+}
+
+QString Addressee::kind() const
+{
+  return d->mKind;
+}
+
 
 void Addressee::setFormattedName( const QString &formattedName )
 {
@@ -1228,32 +1260,44 @@ QString Addressee::fullEmail( const QString &email ) const
   return text;
 }
 
-void Addressee::insertEmail( const QString &email, bool preferred )
+void Addressee::insertEmail( const QString &email, bool preferred, const QMap<QString, QStringList> &param )
 {
-  if ( email.simplified().isEmpty() )
-    return;
-
-  if ( d->mEmails.contains( email ) ) {
-    if ( !preferred || d->mEmails.first() == email )
+    if ( email.simplified().isEmpty() )
       return;
 
-    d->mEmails.removeAll( email );
-    d->mEmails.prepend( email );
-  } else {
+    for (int i = 0; i < d->mEmails.size(); ++i) {
+        if (d->mEmails.at(i).mail() == email)  {
+            if (!preferred || i == 0) {
+                return;
+            }
+            Email tempMail = d->mEmails.takeAt(i);
+            d->mEmails.prepend( tempMail );
+            return;
+        }
+    }
+
+    Email mail(email);
+    mail.setParameters(param);
     d->mEmpty = false;
     if ( preferred ) {
-      d->mEmails.prepend( email );
+        d->mEmails.prepend( mail );
     } else {
-      d->mEmails.append( email );
+        d->mEmails.append( mail );
     }
-  }
+}
+
+void Addressee::insertEmail( const QString &email, bool preferred )
+{
+    insertEmail(email, preferred, QMap<QString, QStringList>());
 }
 
 void Addressee::removeEmail( const QString &email )
 {
-  if ( d->mEmails.contains( email ) ) {
-    d->mEmails.removeAll( email );
-  }
+    for (int i = 0; i < d->mEmails.size(); ++i) {
+        if (d->mEmails.at(i).mail() == email)  {
+            d->mEmails.removeAt(i);
+        }
+    }
 }
 
 QString Addressee::preferredEmail() const
@@ -1261,18 +1305,84 @@ QString Addressee::preferredEmail() const
   if ( d->mEmails.count() == 0 )
     return QString();
   else
-    return d->mEmails.first();
+    return d->mEmails.first().mail();
 }
 
 QStringList Addressee::emails() const
 {
-  return d->mEmails;
+    QStringList list;
+    const int numberOfEmail(d->mEmails.size());
+    for (int i = 0; i < numberOfEmail; ++i) {
+        list << d->mEmails.at(i).mail();
+    }
+
+    return list;
+}
+
+Email::List Addressee::emailList() const
+{
+    return d->mEmails;
+}
+
+void Addressee::setEmailList(const Email::List &list)
+{
+    d->mEmails = list;
 }
 
 void Addressee::setEmails( const QStringList& emails )
 {
-  d->mEmails = emails;
+    d->mEmails.clear();
+    for (int i = 0; i < emails.size(); ++i) {
+        d->mEmails.append(Email(emails.at(i)));
+    }
 }
+
+void Addressee::removeLang( const QString &language )
+{
+    for (int i = 0; i < d->mLangs.size(); ++i) {
+        if (d->mLangs.at(i).language() == language)  {
+            d->mLangs.removeAt(i);
+        }
+    }
+}
+
+void Addressee::insertLang( const Lang &language )
+{
+    const QString languageStr = language.language();
+    if (languageStr.simplified().isEmpty())
+        return;
+    d->mEmpty = false;
+
+    Lang::List::Iterator it;
+    Lang::List::Iterator end(d->mLangs.end());
+    for ( it = d->mLangs.begin(); it != end; ++it ) {
+        if ( ( *it ).language() == languageStr ) {
+            (*it).setParameters(language.parameters());
+            return;
+        }
+    }
+    d->mLangs.append( language );
+}
+
+Lang::List Addressee::langs() const
+{
+    return d->mLangs;
+}
+
+void Addressee::setGender(const Gender &gender)
+{
+    if ( gender == d->mGender )
+      return;
+
+    d->mEmpty = false;
+    d->mGender = gender;
+}
+
+Gender Addressee::gender() const
+{
+    return d->mGender;
+}
+
 void Addressee::insertPhoneNumber( const PhoneNumber &phoneNumber )
 {
   d->mEmpty = false;
@@ -1470,15 +1580,26 @@ QString Addressee::toString() const
   str += QString::fromLatin1( "  Logo: %1\n" ).arg( logo().toString() );
   str += QString::fromLatin1( "  Photo: %1\n" ).arg( photo().toString() );
   str += QString::fromLatin1( "  Sound: %1\n" ).arg( sound().toString() );
+  str += QString::fromLatin1( "  Gender: %1\n" ).arg( gender().toString() );
+  str += QString::fromLatin1( "  Kind: %1\n" ).arg(kind());
 
   str += QLatin1String( "  Emails {\n" );
-  const QStringList e = emails();
-  QStringList::ConstIterator it;
-  for ( it = e.begin(); it != e.end(); ++it ) {
-    str += QString::fromLatin1( "    %1\n" ).arg( *it );
+  const Email::List listEmail = d->mEmails;
+  Email::List::ConstIterator it5;
+  for ( it5 = listEmail.begin(); it5 != listEmail.end(); ++it5 ) {
+    str += ( *it5 ).toString();
   }
   str += QLatin1String( "  }\n" );
 
+  str += QLatin1String( "  Langs {\n" );
+  const Lang::List listLang = d->mLangs;
+  Lang::List::ConstIterator it6;
+  for ( it6 = listLang.begin(); it6 != listLang.end(); ++it6 ) {
+    str += ( *it6 ).toString();
+  }
+  str += QLatin1String( "  }\n" );
+
+
   str += QLatin1String( "  PhoneNumbers {\n" );
   const PhoneNumber::List p = phoneNumbers();
   PhoneNumber::List::ConstIterator it2;
@@ -1915,6 +2036,9 @@ QDataStream &KABC::operator<<( QDataStream &s, const Addressee &a )
   s << a.d->mCategories;
   s << a.customs();
   s << a.d->mKeys;
+  s << a.d->mLangs;
+  s << a.d->mGender;
+  s << a.d->mKind;
   return s;
 }
 
@@ -1955,7 +2079,9 @@ QDataStream &KABC::operator>>( QDataStream &s, Addressee &a )
   s >> customFields;
   a.setCustoms( customFields );
   s >> a.d->mKeys;
-
+  s >> a.d->mLangs;
+  s >> a.d->mGender;
+  s >> a.d->mKind;
   a.d->mEmpty = false;
 
   return s;
