@@ -35,6 +35,9 @@
 #include "kabc/picture.h"
 #include "kabc/secrecy.h"
 #include "kabc/sound.h"
+#include "kabc/email.h"
+#include "kabc/lang.h"
+#include "kabc/gender.h"
 #include "kabc/timezone.h"
 #include "kabc/addresseelist.h"  // for typedef QList<Addressee> List;
 
@@ -698,7 +701,7 @@ class KABC_EXPORT Addressee
        @param list The list of email addresses.
      */
     void setEmails( const QStringList& list);
-
+    void setEmailList(const Email::List &list);
     /**
       Insert a phone number. If a phone number with the same id already exists
       in this addressee it is not duplicated.
@@ -976,7 +979,21 @@ class KABC_EXPORT Addressee
      */
     static QString mimeType();
 
-  private:
+    //kf5 merge with insertEmail(...)
+    void insertEmail(const QString &email, bool preferred, const QMap<QString, QStringList> &param);
+
+    KABC::Email::List emailList() const;
+
+    void removeLang(const QString &language);
+    void insertLang(const Lang &language);
+    Lang::List langs() const;
+
+    void setGender(const KABC::Gender &gender);
+    KABC::Gender gender() const;
+
+    QString kind() const;
+    void setKind(const QString &kind);
+private:
     class Private;
     QSharedDataPointer<Private> d;
 };
