@@ -23,9 +23,12 @@
 #include "picture.h"
 #include "secrecy.h"
 #include "sound.h"
+#include "lang.h"
+#include "gender.h"
 
 #include <QtCore/QString>
 #include <QtCore/QBuffer>
+#include <QDebug>
 
 using namespace KABC;
 
@@ -180,16 +183,61 @@ QByteArray VCardTool::createVCards( const Addressee::List &list,
       card.addLine( createSecrecy( ( *addrIt ).secrecy() ) );
     }
 
+    // LANG only for version == 4.0
+    if ( version == VCard::v4_0 ) {
+        const Lang::List langList = ( *addrIt ).langs();
+        Lang::List::ConstIterator langIt;
+        Lang::List::ConstIterator langEnd( langList.end() );
+        for ( langIt = langList.begin(); langIt != langEnd; ++langIt ) {
+            VCardLine line( QLatin1String( "LANG" ), (*langIt).language() );
+            QMapIterator<QString, QStringList> i((*langIt).parameters());
+            while (i.hasNext()) {
+                i.next();
+                line.addParameter( i.key(), i.value().join(QLatin1String(",")) );
+            }
+            card.addLine( line );
+        }
+    }
     // EMAIL
-    const QStringList emails = ( *addrIt ).emails();
+    const Email::List emailList = ( *addrIt ).emailList();
+    Email::List::ConstIterator emailIt;
+    Email::List::ConstIterator emailEnd( emailList.end() );
     bool pref = true;
-    for ( strIt = emails.begin(); strIt != emails.end(); ++strIt ) {
-      VCardLine line( QLatin1String( "EMAIL" ), *strIt );
-      if ( pref == true && emails.count() > 1 ) {
-        line.addParameter( QLatin1String( "TYPE" ), QLatin1String( "PREF" ) );
-        pref = false;
-      }
-      card.addLine( line );
+
+    for ( emailIt = emailList.begin(); emailIt != emailEnd; ++emailIt ) {
+        bool needToAddPref = false;
+        VCardLine line( QLatin1String( "EMAIL" ), (*emailIt).mail() );
+        if ( pref == true && emailList.count() > 1 ) {
+            needToAddPref = true;
+            pref = false;
+        }
+        QMapIterator<QString, QStringList> i((*emailIt).parameters());
+        bool foundType = false;
+        while (i.hasNext()) {
+            i.next();
+            QStringList valueStringList = i.value();
+            if (i.key().toLower() == QLatin1String( "type" )) {
+                if (!valueStringList.contains(QLatin1String("PREF"))) {
+                    if (needToAddPref) {
+                        valueStringList.append(QLatin1String( "PREF" ));
+                    } else {
+                        needToAddPref = false;
+                    }
+                } else {
+                    if (!needToAddPref) {
+                        valueStringList.removeAll(QLatin1String( "PREF" ));
+                    }
+                }
+                foundType = true;
+            }
+            if (!valueStringList.isEmpty()) {
+                line.addParameter( i.key(), valueStringList.join(QLatin1String(",")) );
+            }
+        }
+        if (!foundType && needToAddPref) {
+            line.addParameter( QLatin1String( "TYPE" ), QLatin1String( "PREF" ) );
+        }
+        card.addLine( line );
     }
 
     // FN required for only version > 2.1
@@ -204,7 +252,11 @@ QByteArray VCardTool::createVCards( const Addressee::List &list,
     const Geo geo = ( *addrIt ).geo();
     if ( geo.isValid() ) {
       QString str;
-      str.sprintf( "%.6f;%.6f", geo.latitude(), geo.longitude() );
+      if (version == VCard::v4_0 ) {
+          str.sprintf( "geo:%.6f,%.6f", geo.latitude(), geo.longitude() );
+      } else {
+          str.sprintf( "%.6f;%.6f", geo.latitude(), geo.longitude() );
+      }
       card.addLine( VCardLine( QLatin1String( "GEO" ), str ) );
     }
 
@@ -213,11 +265,11 @@ QByteArray VCardTool::createVCards( const Addressee::List &list,
     Key::List::ConstIterator keyIt;
     Key::List::ConstIterator keyEnd( keys.end() );
     for ( keyIt = keys.begin(); keyIt != keyEnd; ++keyIt ) {
-      card.addLine( createKey( *keyIt ) );
+      card.addLine( createKey( *keyIt, version ) );
     }
 
     // LOGO
-    card.addLine( createPicture( QLatin1String( "LOGO" ), ( *addrIt ).logo() ) );
+    card.addLine( createPicture( QLatin1String( "LOGO" ), ( *addrIt ).logo(), version ) );
 
     // MAILER only for version < 4.0
     if ( version != VCard::v4_0 ) {
@@ -296,7 +348,7 @@ QByteArray VCardTool::createVCards( const Addressee::List &list,
     card.addLine( orgLine );
 
     // PHOTO
-    card.addLine( createPicture( QLatin1String( "PHOTO" ), ( *addrIt ).photo() ) );
+    card.addLine( createPicture( QLatin1String( "PHOTO" ), ( *addrIt ).photo(), version ) );
 
     // PROID only for version > 2.1
     if ( version != VCard::v2_1 ) {
@@ -320,7 +372,7 @@ QByteArray VCardTool::createVCards( const Addressee::List &list,
     }
 
     // SOUND
-    card.addLine( createSound( ( *addrIt ).sound() ) );
+    card.addLine( createSound( ( *addrIt ).sound(), version ) );
 
     // TEL
     const PhoneNumber::List phoneNumbers = ( *addrIt ).phoneNumbers();
@@ -381,7 +433,7 @@ QByteArray VCardTool::createVCards( const Addressee::List &list,
     }
 
     // X-
-    const QStringList customs = ( *addrIt ).customs();
+    const QStringList customs = ( *addrIt ).customs();    
     for ( strIt = customs.begin(); strIt != customs.end(); ++strIt ) {
       QString identifier = QLatin1String( "X-" ) +
                                  ( *strIt ).left( ( *strIt ).indexOf( QLatin1Char( ':' ) ) );
@@ -415,16 +467,46 @@ QByteArray VCardTool::createVCards( const Addressee::List &list,
           identifier = QLatin1String( "X-IRC" ); //Not defined by rfc but need for fixing #300869
         } else if ( identifier == QLatin1String( "X-messaging/googletalk-All" ) ) {
           //Not defined by rfc but need for fixing #300869
-          identifier = QLatin1String( "X-GOOGLETALK" );
+          identifier = QLatin1String( "X-GTALK" );
         }
       }
 
-      VCardLine line( identifier, value );
-      if ( version == VCard::v2_1 && needsEncoding( value ) ) {
-        line.addParameter( QLatin1String( "charset" ), QLatin1String( "UTF-8" ) );
-        line.addParameter( QLatin1String( "encoding" ), QLatin1String( "QUOTED-PRINTABLE" ) );
+      if (version == VCard::v4_0) {
+          // GENDER          
+          const Gender gender = ( *addrIt ).gender();
+          if (gender.isValid()) {
+              QString genderStr;
+              if (!gender.gender().isEmpty()) {
+                  genderStr = gender.gender();
+              }
+              if (!gender.comment().isEmpty()) {
+                  genderStr += QLatin1Char(';') + gender.comment();
+              }
+              VCardLine line( QLatin1String( "GENDER" ), genderStr );
+              card.addLine( line );
+          }
+          // KIND
+          if (!( *addrIt ).kind().isEmpty()) {
+              VCardLine line( QLatin1String( "KIND" ), ( *addrIt ).kind() );
+              card.addLine(line);
+          }
+      }
+      if (identifier.toLower() == QLatin1String( "x-kaddressbook-x-anniversary" ) && version == VCard::v4_0) {
+          // ANNIVERSARY
+          if (!value.isEmpty()) {
+              const QDate date = QDate::fromString( value, Qt::ISODate );
+              QDateTime dt = QDateTime(date);
+              dt.setTime(QTime());
+              card.addLine( VCardLine( QLatin1String( "ANNIVERSARY" ), createDateTime( dt ) ) );
+          }
+      } else {
+          VCardLine line( identifier, value );
+          if ( version == VCard::v2_1 && needsEncoding( value ) ) {
+              line.addParameter( QLatin1String( "charset" ), QLatin1String( "UTF-8" ) );
+              line.addParameter( QLatin1String( "encoding" ), QLatin1String( "QUOTED-PRINTABLE" ) );
+          }
+          card.addLine( line );
       }
-      card.addLine( line );
     }
 
     vCardList.append( card );
@@ -502,7 +584,12 @@ Addressee::List VCardTool::parseVCards( const QByteArray &vcard ) const
         else if ( identifier == QLatin1String( "bday" ) ) {
           addr.setBirthday( parseDateTime( ( *lineIt ).value().toString() ) );
         }
-
+        // ANNIVERSARY
+        else if ( identifier == QLatin1String( "anniversary" ) ) {
+            const QString t = ( *lineIt ).value().toString();
+            const QDateTime dt(parseDateTime( t ));
+            addr.insertCustom( QLatin1String("KADDRESSBOOK"), QLatin1String("X-Anniversary"), dt.date().toString(Qt::ISODate) );
+        }
         // CATEGORIES
         else if ( identifier == QLatin1String( "categories" ) ) {
           const QStringList categories = splitString( commaSep, ( *lineIt ).value().toString() );
@@ -518,9 +605,36 @@ Addressee::List VCardTool::parseVCards( const QByteArray &vcard ) const
         else if ( identifier == QLatin1String( "email" ) ) {
           const QStringList types = ( *lineIt ).parameters( QLatin1String( "type" ) );
           addr.insertEmail( ( *lineIt ).value().toString(),
-                            types.contains( QLatin1String( "PREF" ) ) );
+                            types.contains( QLatin1String( "PREF" ) ), (*lineIt).parameterMap() );
+        }
+        // GENDER
+        else if ( identifier == QLatin1String( "gender" ) ) {
+            QString genderStr = ( *lineIt ).value().toString();
+            if (!genderStr.isEmpty()) {
+                Gender gender;
+                if (genderStr.at(0) != QLatin1Char(';')) {
+                    gender.setGender(genderStr.at(0));
+                    if (genderStr.length() > 2) {
+                        gender.setComment(genderStr.right(genderStr.length()-2));
+                    }
+                } else {
+                    gender.setComment(genderStr.right(genderStr.length()-1));
+                }
+                addr.setGender(gender);
+            }
         }
 
+        // LANG
+        else if ( identifier == QLatin1String( "lang" ) ) {
+            Lang lang;
+            lang.setLanguage(( *lineIt ).value().toString());
+            lang.setParameters((*lineIt).parameterMap());
+            addr.insertLang(lang);
+        }
+        // KIND
+        else if ( identifier == QLatin1String( "kind" ) ) {
+          addr.setKind( ( *lineIt ).value().toString() );
+        }
         // FN
         else if ( identifier == QLatin1String( "fn" ) ) {
           addr.setFormattedName( ( *lineIt ).value().toString() );
@@ -529,13 +643,24 @@ Addressee::List VCardTool::parseVCards( const QByteArray &vcard ) const
         // GEO
         else if ( identifier == QLatin1String( "geo" ) ) {
           Geo geo;
-
-          const QStringList geoParts =
-            ( *lineIt ).value().toString().split( QLatin1Char( ';' ), QString::KeepEmptyParts );
-          if ( geoParts.size() >= 2 ) {
-            geo.setLatitude( geoParts.at( 0 ).toFloat() );
-            geo.setLongitude( geoParts.at( 1 ).toFloat() );
-            addr.setGeo( geo );
+          QString lineStr = ( *lineIt ).value().toString();
+          if (lineStr.startsWith(QLatin1String("geo:"))) { //VCard 4.0
+              lineStr.remove(QLatin1String("geo:"));
+              const QStringList geoParts =
+                lineStr.split( QLatin1Char( ',' ), QString::KeepEmptyParts );
+              if ( geoParts.size() >= 2 ) {
+                geo.setLatitude( geoParts.at( 0 ).toFloat() );
+                geo.setLongitude( geoParts.at( 1 ).toFloat() );
+                addr.setGeo( geo );
+              }
+          } else {
+              const QStringList geoParts =
+                      lineStr.split( QLatin1Char( ';' ), QString::KeepEmptyParts );
+              if ( geoParts.size() >= 2 ) {
+                  geo.setLatitude( geoParts.at( 0 ).toFloat() );
+                  geo.setLongitude( geoParts.at( 1 ).toFloat() );
+                  addr.setGeo( geo );
+              }
           }
         }
 
@@ -757,7 +882,7 @@ Addressee::List VCardTool::parseVCards( const QByteArray &vcard ) const
             ident = QLatin1String( "X-messaging/meanwhile-All" );
           } else if ( identifier == QLatin1String( "x-irc" ) ) {
             ident = QLatin1String( "X-messaging/irc-All" );
-          } else if ( identifier == QLatin1String( "x-googletalk" ) ) {
+          } else if ( identifier == QLatin1String( "x-gtalk" ) ) {
             ident = QLatin1String( "X-messaging/googletalk-All" );
           }
 
@@ -787,7 +912,6 @@ QDateTime VCardTool::parseDateTime( const QString &str ) const
     date = QDate( str.left( 4 ).toInt(), str.mid( 5, 2 ).toInt(),
                   str.mid( 8, 2 ).toInt() );
   }
-
   // does it also contain a time ? (Note: mm, ss are optional according ISO-8601)
   int timeStart = str.indexOf( QLatin1Char( 'T' ) );
   if ( timeStart >= 0 ) {
@@ -872,8 +996,9 @@ Picture VCardTool::parsePicture( const VCardLine &line ) const
   return pic;
 }
 
-VCardLine VCardTool::createPicture( const QString &identifier, const Picture &pic ) const
+VCardLine VCardTool::createPicture( const QString &identifier, const Picture &pic, VCard::Version version ) const
 {
+  Q_UNUSED(version);
   VCardLine line( identifier );
 
   if ( pic.isEmpty() ) {
@@ -913,8 +1038,9 @@ Sound VCardTool::parseSound( const VCardLine &line ) const
   return snd;
 }
 
-VCardLine VCardTool::createSound( const Sound &snd ) const
+VCardLine VCardTool::createSound( const Sound &snd, VCard::Version version ) const
 {
+  Q_UNUSED(version);
   VCardLine line( QLatin1String( "SOUND" ) );
 
   if ( snd.isIntern() ) {
@@ -956,8 +1082,9 @@ Key VCardTool::parseKey( const VCardLine &line ) const
   return key;
 }
 
-VCardLine VCardTool::createKey( const Key &key ) const
+VCardLine VCardTool::createKey( const Key &key, VCard::Version version ) const
 {
+  Q_UNUSED(version);
   VCardLine line( QLatin1String( "KEY" ) );
 
   if ( key.isBinary() ) {
