@@ -26,7 +26,7 @@ class TextEditTester : public QObject
 {
   Q_OBJECT
 
-  private slots:
+  private Q_SLOTS:
     void testFormattingUsed();
     void testQuoting();
     void testCleanText();
