@@ -22,7 +22,6 @@
 
 #include <QObject>
 
-class KJob;
 
 /**
   This class uses the DispatcherInterface to send an abort() signal th the MDA.
@@ -34,7 +33,7 @@ class Runner : public QObject
   public:
     Runner();
 
-  private slots:
+  private Q_SLOTS:
     void sendAbort();
 
 };
